<?php

declare(strict_types=1);

namespace CreoCode\ContaoTourismBundle\Picker;

use Contao\CoreBundle\Framework\FrameworkAwareInterface;
use Contao\CoreBundle\Framework\FrameworkAwareTrait;
use Contao\CoreBundle\Picker\AbstractPickerProvider;
use Contao\CoreBundle\Picker\DcaPickerProviderInterface;
use Contao\CoreBundle\Picker\PickerConfig;

use CreoCode\ContaoTourismBundle\AccommodationsSetModel;
use CreoCode\ContaoTourismBundle\AccommodationsModel;

use Knp\Menu\FactoryInterface;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\Security\Core\Security;
use Symfony\Component\Translation\TranslatorInterface;

class AccommodationsPickerProvider extends AbstractPickerProvider implements DcaPickerProviderInterface, FrameworkAwareInterface
{
    use FrameworkAwareTrait;

    /**
     * {@inheritdoc}
     */
    public function getName(): string
    {
        return 'accommodationsPicker';
    }

    /**
     * {@inheritdoc}
     */
    public function supportsContext($context): bool
    {
        return 'link' === $context && $this->getUser()->hasAccess('accommodations', 'modules');
    }

    /**
     * {@inheritdoc}
     */
    public function supportsValue(PickerConfig $config): bool
    {
        return false !== strpos($config->getValue(), '{{accommodations_url::');
    }

    /**
     * {@inheritdoc}
     */
    public function getDcaTable(): string
    {
        return 'tl_accommodations';
    }

    /**
     * {@inheritdoc}
     */
    public function getDcaAttributes(PickerConfig $config): array
    {
        $attributes = ['fieldType' => 'radio'];

        if ($source = $config->getExtra('source')) {
            $attributes['preserveRecord'] = $source;
        }

        if ($this->supportsValue($config)) {
            $attributes['value'] = str_replace(['{{accommodations_url::', '}}'], '', $config->getValue());
        }

        return $attributes;
    }

    /**
     * {@inheritdoc}
     */
    public function convertDcaValue(PickerConfig $config, $value): string
    {
        return '{{accommodations_url::'.$value.'}}';
    }

    /**
     * {@inheritdoc}
     */
    protected function getRouteParameters(PickerConfig $config = null): array
    {
        $params = ['do' => 'accommodations'];

        if (null === $config || !$config->getValue() || false === strpos($config->getValue(), '{{accommodations_url::')) {
            return $params;
        }

        $value = str_replace(['{{accommodations_url::', '}}'], '', $config->getValue());

        if (null !== ($accommodationsSetId = $this->getAccommodationsSetId($value))) {
            $params['table'] = 'tl_accommodations';
            $params['id'] = $accommodationsSetId;
        }

        return $params;
    }

    /**
     * @param int|string $id
     */
    private function getAccommodationsSetId($id): ?int
    {
        /** @var AccommodationsModel $accommodationsAdapter */
        $accommodationsAdapter = $this->framework->getAdapter(AccommodationsModel::class);
        $accommodationsSetAdapter = $this->framework->getAdapter(AccommodationsSetModel::class);

        if (!($accommodationsModel = $accommodationsAdapter->findById($id)) instanceof AccommodationsModel) {
            return null;
        }

        //if (!($accommodationsSet = $accommodationsModel->getRelated('pid')) instanceof AccommodationsSetModel) {
        if (!($accommodationsSet = $accommodationsSetAdapter->findById($accommodationsModel->pid)) instanceof AccommodationsSetModel) {
            return null;
        }

        return (int) $accommodationsSet->id;
    }
}
